* ------------------------------------------------------
*  Special Function Registers

       .include "sfr.i"

* ------------------------------------------------------
*  Variable-Definitions

;Oscillation Control Register settings
;Internal RC oscillation for system clock
osc_rc EQU 77
;Crystal oscillation for system clock
osc_xt EQU 239

* ------------------------------------------------------
*  Interupt Vectors

	.include "libraries/intvec.i"

* ------------------------------------------------------
*  Game Data

	.include "data/gamehdr.i"
	.include "data/iconanm.i"
	.include "data/iconplt.i"
	.include "data/icondat.i"

* ------------------------------------------------------
*  Main Program Loop

	.include "libraries/main.i"

* ------------------------------------------------------
*  Jump To User Program

	;jmp begin

* ------------------------------------------------------
*  User Defined Imports


* ------------------------------------------------------
*  User Defined Program

begin:
	;Clears the LCD display image
	call cls
	;P3 initialization (pull-up setting)
	mov #$ff, P3

loop0:
	;[A] Button Check
	mov #0, b
	;Loads the status of P3
	ld P3
	;next3 if [A] button is being pressed
	bp acc, 4, next3
	;Display character code 'A'
	mov #1, b

next3:
	ld b
	;Display coordinate (horizontal)
	mov #4, c
	;Display coordinate (vertical)
	mov #3, b
	;Displays single character
	call putch

	;[B] Button Check
	mov #0, b
	ld P3
	;next4 if [B] button is being pressed
	bp acc, 5, next4
	;Display character code 'B'
	mov #2, b

next4:
	ld b
	mov #5, c
	mov #2, b
	call putch
	
	;[↑] Button Check
	mov #0, b
	ld P3
	;next5 if [↑] button is being pressed
	bp acc, 0, next5
	;Display character code '↑'
	mov #3, b

next5:
	ld b
	mov #1, c
	mov #1, b
	call putch

	;[→] Button Check	
	mov #0, b
	ld P3
	;next6 if [→] button is being pressed
	bp acc, 3, next6
	;Display character code '→'
	mov #4, b

next6:
	ld b
	mov #2, c
	mov #2, b
	call putch
	
	;[↓] Button Check
	mov #0, b
	ld P3
	;next7 if [↓] button is being pressed
	bp acc, 1, next7
	;Display character code '↓'
	mov #5, b

next7:
	ld b
	mov #1, c
	mov #3, b
	call putch

	;[←] Button Check	
	mov #0, b
	ld P3
	;next8 if [←] button is being pressed
	bp acc, 2, next8
	;Display character code '←'
	mov #6, b

next8:
	ld b
	mov #0, c
	mov #2, b
	call putch
	
	;[S] Button Check
	mov #0, b
	ld P3
	;next9 if [S] button is being pressed
	bp acc, 7, next9
	;Display character code 'S'
	mov #8, b
	
next9:
	ld b
	mov #4, c
	mov #1, b
	call putch
	
	;[M] Button Check
	ld p3
	;If the [M] button is pressed, the application ends
	bn acc, 6, .finish
	;Repeat
	brf loop0
.finish:
	call finish

;End program
finish:
	jmpf goodbye

;Clears the LCD display image
cls:
	;Pushes the OCR value onto the stack
	push OCR
	;Specifies the system clock
	mov #osc_rc, OCR
	;Specifies the display RAM bank address (BANK0)
	mov #0, XBNK
	;Clears the data in that bank
	call cls_s
	;Specifies the display RAM bank address (BANK1)
	mov #1, XBNK
	;Clears the data in that bank
	call cls_s
	;Pops the OCR value onto the stack
	pop OCR
	;Cls end
	ret

;Clearing one bank of display RAM
cls_s:
	;Points the indirect addressing register at the start of display RAM
	mov #$80, 2
	;Sets the number of loops in loop counter b
	mov #$80, b
loop3:
	;Writes "0" while incrementing the address
	mov #0, @r2
	inc 2
	;Repeats until b is "0"
	dbnz b, loop3
	;Cls_s end
	ret

;Displaying one character at a specified position
;Inputs
;	acc: Character code
;	c: Horizontal position of character
;	b: Vertical position of character
putch:
	push XBNK
	push acc
	;Calculates display RAM address according to coordinates
	call locate
	pop acc
	;Displays one character
	call put_chara
	pop XBNK
	;Putch end
	ret

;Calculating the display RAM address according to the display position specification
;Inputs
;	c: Horizontal position (0 to 5)
;	b: Vertical position (0 to 3)
;Outputs
;	2: RAM address
;	XBNK: display RAM bank
;Determining the display RAM bank address
locate:
	;Jump to next1 when b >= 2
	ld b
	sub #2
	bn PSW, 7, next1
	;Specifies the display RAM bank address (BANK0)
	mov #$0, XBNK
	br next2

next1:
	st b
	;Specifies the display RAM bank address (BANK1)
	mov #$01, XBNK

;Calculating the RAM address for a specified position on the display
next2:
	;b * 40h + c + 80h
	ld b
	rol
	rol
	rol
	rol
	rol
	rol
	add c
	add #$80
	;Stores the RAM address in 2
	st 2
	;Locate end
	ret

put_chara:
	;Pushes the PSW value onto the stack
	push PSW
	;Selects data RAM bank 1
	set1 PSW, 1
	;Calculating the character data address
	;(TRH,TRL) = acc*8 + fontdata
	rol
	rol
	rol
	add #<fontdata
	st TRL
	mov #0, acc
	addc #>fontdata
	st TRH
	;Pushes the OCR value onto the stack
	push OCR
	;Specifies the system clock
	mov #osc_rc, OCR
	;Offset value for loading the character data
	mov #0, b
	;Loop counter
	mov #4, c
loop1:
	;Loads the display data for the first line
	ld b
	ldc
	;Increments the load data offset by 1
	inc b
	;Transfers the display data to display RAM
	st @r2
	ld 2
	;Adds 6 to the display RAM address
	add #6
	st 2

	;Loads the display data for the second line
	ld b
	ldc
	;Increments the load data offset by 1
	inc b
	;Transfers the display data to display RAM
	st @r2
	ld 2
	;Adds 10 to the display RAM address
	add #10
	st 2

	;Decrements the loop counter
	dec c
	ld c
	;Repeats for 8 lines (four times)
	bnz loop1
	;Pops the OCR value off of the stack
	pop OCR
	;Pops the PSW value off of the stack
	pop PSW
	;Put_chara end
	ret

;Character bit image data
fontdata:
	.byte $00, $00, $38, $38, $38, $00, $00, $00	; •
	.byte $1e, $36, $66, $c6, $c6, $fe, $c6, $00	; A
	.byte $fc, $66, $66, $7c, $66, $66, $fc, $00	; B
	
	.byte $10, $38, $7c, $fe, $38, $38, $38, $00	; ↑
	.byte $10, $18, $fc, $fe, $fc, $18, $10, $00	; →
	.byte $38, $38, $38, $fe, $7c, $38, $10, $00	; ↓
	.byte $10, $30, $7e, $fe, $7e, $30, $10, $00	; ←
	
	.byte $c6, $ee, $fe, $d6, $c6, $c6, $c6, $00	; M
	.byte $7c, $e6, $76, $38, $dc, $ce, $7c, $00	; S

* ------------------------------------------------------
*  End Program

	.include "libraries/endprog.i"

