* ------------------------------------------------------
*  Special Function Registers

       .include "sfr.i"

* ------------------------------------------------------
*  Variable-Definitions

;Oscillation Control Register settings
;Internal RC oscillation for system clock
osc_rc EQU 129
;Crystal oscillation for system clock
osc_xt EQU 130

* ------------------------------------------------------
*  Interupt Vectors

	.include "libraries/intvec.i"

* ------------------------------------------------------
*  Game Data

	.include "data/gamehdr.i"
	.include "data/iconanm.i"
	.include "data/iconplt.i"
	.include "data/icondat.i"

* ------------------------------------------------------
*  Main Program Loop

	.include "libraries/main.i"

* ------------------------------------------------------
*  Jump To User Program

	;jmp begin

* ------------------------------------------------------
*  User Defined Imports


* ------------------------------------------------------
*  User Defined Program

begin:
	;Sound initialization
	call SndInit

loop0:
	;Starts generating tone at approximately 342Hz
	call Snd1Start
	;HALT mode until base timer interrupt (0.5 seconds)
	set1 PCON, 0

	;Buzzer sound off	
	call SndStop
	;HALT mode until base timer interrupt (0.5 seconds)
	set1 PCON, 0

	;Starts generating tone at approximately 781
	call Snd2Start
	;HALT mode until base timer interrupt (0.5 seconds)
	set1 PCON, 0
	
	;Buzzer sound off
	call SndStop
	;HALT mode until base timer interrupt (0.5 seconds)
	set1 PCON, 0

	;[M] (mode) Button Check
	ld P3
	;If the [M] button is pressed, the application ends
	bn ACC, 6, .finish

	;Repeat
	br loop0
.finish:
	call finish

;End program
finish:
	jmpf goodbye

;Sound output-related routines

;Sound output hardware initialization
SndInit:
	;Sets the sound output port
	clr1 P1, 7
	ret

;Start of 342Hz tone
Snd1Start:
	;Cycles = 100h - 0f0h = 16
	mov #$f0, T1LR
	;L level width = 100h - 0f8h = 8
	mov #$f8, T1LC
	;Sound output start
	mov #$d0, T1CNT
	ret

;Start of 781Hz tone
Snd2Start:
	;Cycles = 100h - 0f9h = 7
	mov #$f9, T1LR
	;L level width = 100h - 0fch = 4
	mov #$fc, T1LC
	;Sound output start
	mov #$d0, T1CNT
	ret

;Sound stop
SndStop:
	;Stops sound output
	mov #0, T1CNT
	ret

* ------------------------------------------------------
*  End Program

	.include "libraries/endprog.i"
