;Timer/Counter T0 Interrupt Usage Sample 1
;Intermittently sounds the buzzer (every two seconds)

* ------------------------------------------------------
*  Special Function Registers

       .include "sfr.i"

* ------------------------------------------------------
*  Variable-Definitions

;Oscillation Control Register settings
;Internal RC oscillation for system clock
osc_rc EQU 77
;Crystal oscillation for system clock
osc_xt EQU 239

;Timer interrupt counter
counter = $30

* ------------------------------------------------------
*  Interupt Vectors

	.include "libraries/intvec.i"

* ------------------------------------------------------
*  Game Data

	.include "data/gamehdr.i"
	.include "data/iconanm.i"
	.include "data/iconplt.i"
	.include "data/icondat.i"

* ------------------------------------------------------
*  Main Program Loop

	.include "libraries/main.i"

* ------------------------------------------------------
*  Jump To User Program

	;jmp begin

* ------------------------------------------------------
*  User Defined Imports


* ------------------------------------------------------
*  User Defined Program

begin:
	;Sound output initialization
	call SndInit
	;Timer T0 initialization
	call T0Mode2Init
	;Clears counter
	mov #0, counter

loop0:
	;Loads the counter value
	ld counter
	;next1 if bit 0 of the counter is "1"
	bp ACC, 1, next1

	;Starts sound
	call Snd2Start
	br next2

next1:
	;Stops sound
	call SndStop

next2:
	;HALT mode until next interrupt
	set1 PCON, 0
	;[M] (mode) Button Check
	ld P3
	;If the [M] button is pressed, the application ends
	bn acc, 6, .finish
	;Repeat
	br loop0
.finish:
	call finish

;End program
finish:
	jmpf goodbye

;Timer/counter T0 initialization
;Applied an interrupt about once per second in mode 2 (16-bit counter)
T0Mode2Init:
	;Sets the prescaler value
	;Since this is an 8-bit prescaler:
	;Cycle = (256-255) * 0.000183 = 0.000183 (sec.)
	mov #255, T0PRR
	;Sets preset value (H)
	mov #>(65536-5464), T0HR
	;Sets preset value (L)
	mov #<(65536-5464), T0LR

	;As a set with the prescaler:
	;0.000183 * 5464 = 0.999912 (a1sec)
	;An overflow occurs about once per second

	;Sets up an immediate initial overflow
	mov #$ff, T0L
	mov #$ff, T0H
	;Mode 2 (16-bit counter)
	;Generates an interrupt according to the T0H overflow
	;T0 operation start
	mov #$e4, T0CON
	;T0Mode2Init end
	ret

;T0H timer stop
T0HStop:
	;T0H count operation stop
	clr1 T0CON, 7
	ret

;T0H interrupt handler
int_T0H:
	inc counter
	;Clears the timer T0H interrupt source
	clr1 T0CON, 3
	reti

;Sound output-related routines

;Sound output hardware initialization
SndInit:
	;Sets the sound output port
	clr1 P1, 7
	ret

;Start of 342Hz tone
Snd1Start:
	;Cycles = 100h - 0f0h = 16
	mov #$f0, T1LR
	;L level width = 100h - 0f8h = 8
	mov #$f8, T1LC
	;Sound output start
	mov #$d0, T1CNT
	ret

;Start of 781Hz tone
Snd2Start:
	;Cycles = 100h - 0f9h = 7
	mov #$f9, T1LR
	;L level width = 100h - 0fch = 4
	mov #$fc, T1LC
	;Sound output start
	mov #$d0, T1CNT
	ret

;Sound stop
SndStop:
	;Stops sound output
	mov #0, T1CNT
	ret

* ------------------------------------------------------
*  End Program

	.include "libraries/endprog.i"
